
codes_with_steps = [
    ("BR-CS", 5.6),
    ("CA-ON", 9.75),
    ("CH", 1),
    ("DE", 1.38),
    ("PL", 1.25),
    ("BE", 1),
    ("IT-NO", 1.2),
    ("CA-QC", 9.75),
    ("ES", 1.5),
    ("GB", 1.25),
    ("FI", 1.5),
    ("FR", 1.6),
    ("NL", 1),
]


code_names = ["BR-CS", "CA-ON", "CH", "DE", "PL", "BE", "IT-NO", "CA-QC", "ES", "GB", "FI", "FR", "NL"]


codes_to_gcloud_zones = {
    "BR-CS" : 'southamerica-east1-b', 
    "CA-ON" : 'northamerica-northeast2-b', 
    "CH" : 'europe-west6-b', 
    "DE" : 'europe-west3-b', 
    "PL" : 'europe-central2-b', 
    "BE" : 'europe-west1-b', 
    "IT-NO" : 'europe-west8-a', 
    "CA-QC" : 'northamerica-northeast1-c', 
    "ES" : 'europe-southwest1-c', 
    "GB" : 'europe-west2-b', 
    "FI" : 'europe-north1-b', 
    "FR" : 'europe-west9-b', 
    "NL" : 'europe-west4-b' 
}